
__all__ = [

    'UNSPECIFIED',
    'VIOLATION',
    'ALERT',
    'CRITICAL',
    'FAILURE',
    'WARNING',
    'NOTICE',
    'INFORMATIONAL',
    'DEBUG0',
    'DEBUG1',
    'DEBUG2',
    'DEBUG3',
    'DEBUG4',
    'DEBUG5',
    'TRACE',

    'FAIL',
    'WARN',

    'severity_to_string',
]

UNSPECIFIED     =   0
VIOLATION       =   1
ALERT           =   2
CRITICAL        =   3
FAILURE         =   4
WARNING         =   5
NOTICE          =   6
INFORMATIONAL   =   7
DEBUG0          =   8
DEBUG1          =   9
DEBUG2          =   10
DEBUG3          =   11
DEBUG4          =   12
DEBUG5          =   13
TRACE           =   14

FAIL            =   FAILURE
WARN            =   WARNING
INFO            =   INFORMATIONAL

_STOCK_SEVERITY_LEVELS = {

    VIOLATION       :   'VIOLATION',
    ALERT           :   'ALERT',
    CRITICAL        :   'CRITICAL',
    FAILURE         :   'FAILURE',
    WARNING         :   'WARNING',
    NOTICE          :   'NOTICE',
    INFORMATIONAL   :   'INFORMATIONAL',
    DEBUG0          :   'DEBUG0',
    DEBUG1          :   'DEBUG1',
    DEBUG2          :   'DEBUG2',
    DEBUG3          :   'DEBUG3',
    DEBUG4          :   'DEBUG4',
    DEBUG5          :   'DEBUG5',
    TRACE           :   'TRACE',
}



def severity_to_string(severity):

    s = _STOCK_SEVERITY_LEVELS.get(severity, None)

    if s is None:

        if isinstance(severity, (int, long, )):

            s = str(severity)
        else:

            s = "%s (%s)" % (str(severity), type(severity))

        return "<Severity: %s>" % s
    else:

        return s






#!/usr/bin/env python

import diagnosticism as d
import diagnosticism.severity as sev
import logging

d.enable_logging(True)

d.log(sev.TRACE, 'trace')
d.log(sev.INFO, 'info')
d.log(sev.DEBUG2, 'debug2')
d.log(sev.ALERT, "and we're out")





from datetime import datetime as dt
import os
import sys
import threading

from .conrep import report, _isatty
from .program_name import get_program_name
from . import severity as _severity

class _clrs:

    RED         =   '\x1B[31m'
    GREEN       =   '\x1B[32m'
    YELLOW      =   '\x1B[33m'
    BLUE        =   '\x1B[34m'
    MAGENTA     =   '\x1B[35m'
    DARKGREY    =   '\x1B[90m'
    NONE        =   '\x1B[0m'

    D   =   {

        _severity.VIOLATION       :   RED,
        _severity.ALERT           :   RED,
        _severity.CRITICAL        :   RED,
        _severity.FAILURE         :   RED,
        _severity.WARNING         :   YELLOW,
        _severity.NOTICE          :   MAGENTA,
        _severity.INFORMATIONAL   :   MAGENTA,
        _severity.DEBUG0          :   BLUE,
        _severity.DEBUG1          :   BLUE,
        _severity.DEBUG2          :   BLUE,
        _severity.DEBUG3          :   BLUE,
        _severity.DEBUG4          :   BLUE,
        _severity.DEBUG5          :   BLUE,
        _severity.TRACE           :   DARKGREY,
    }


_logging_is_enabled = False

def enable_logging(is_enabled):

    global _logging_is_enabled

    _logging_is_enabled = is_enabled

def is_logging_enabled():

    return _logging_is_enabled

def do_log(severity, message):

    now = dt.now()

    ss = _severity.severity_to_string(severity)

    if _isatty():

        r = _clrs.D.get(severity, None)

        if r:

            ss = r + ss + _clrs.NONE

    prefix = "[%s, %s, %s, %s]" % (get_program_name(), threading.current_thread().ident, str(now), ss)

    full = prefix + ': ' + message

    report(full, show_program_name=False)

def log(severity, message):

    if not _logging_is_enabled:

        return

    return do_log(severity, message)





from .program_name import get_program_name

import os
import sys

_STDERR_ISATTY = sys.stderr.isatty()
_OS_IS_POSIX = os.name.lower() == 'posix'

def _emit_to_cr_stm(message):

    sys.stderr.write(message)

def _add_eol_and_emit_to_cr_stm(message):

    _emit_to_cr_stm(message + "\n")

def _isatty():

    return _STDERR_ISATTY and _OS_IS_POSIX

def conrep(message, **kwargs):

    """DEPRECATED: use report()"""

    return report(message, **kwargs)

def report(message, show_program_name=True):

    """Emits the given message on the contingent report stream. If the optional argument show_program_name is given and is falsey, the program name will not prefix the message"""

    if show_program_name:

        pn = get_program_name()

        _emit_to_cr_stm(pn + ": " + message + "\n")
    else:

        _add_eol_and_emit_to_cr_stm(message)

def abort(message, do_exit=True, show_program_name=True):

    """Emits the program prefix and the given message on the contingent report stream, and then terminates the process"""

    conrep(message, show_program_name=show_program_name)

    if do_exit:
        sys.exit(1)





from .log import do_log
from . import severity

import inspect

_tracingEnabled =   False

def _derive_param(pname, params):

    val = params[pname]
    typ = val.__class__.__name__

    return (pname, typ, val)

def _log_s(severity, message):

    do_log(severity, message)

def enable_tracing(enabled):

    global _tracingEnabled

    _tracingEnabled = enabled

def is_tracing_enabled():

    return _tracingEnabled

def trace():

    if _tracingEnabled:

        fr = inspect.currentframe()

        try:

            fr = fr.f_back

            code = fr.f_code

            fname = code.co_name
            vnames = code.co_varnames
            params = fr.f_locals

            pnames = [n for n in vnames if n in params]

            # Algorithm informed by http://blog.mclemon.io/python-efficient-string-concatenation-in-python-2016-edition

            if 0 != len(params):

                pname0 = pnames[0]

                if 'self' == pname0:

                    val0 = params[pname0]
                    typ0 = val0.__class__.__name__

                    plist = ", ".join(["%s(%s)=%s" % _derive_param(n, params) for n in pnames[1:]])

                    _log_s(severity.TRACE, "%s.%s(%s)" % (typ0, fname, plist))

                    return

            plist = ", ".join(["%s(%s)=%s" % _derive_param(n, params) for n in pnames])

            _log_s(severity.TRACE, "%s(%s)" % (fname, plist))
        finally:

            del fr

